package io.tswf.minecraft.blockventure.core.transmutation

import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.AtExit
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.ClassReference
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.MethodDescription
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.Transmute
import io.tswf.groovy.bettergroovy.asm.transmutation.meta.MissingTargetStrategy
import io.tswf.minecraft.blockventure.core.network.packet.NetworkPlayPacketWrapper
import io.tswf.minecraft.blockventure.core.network.packet.NetworkPlayPacketWrapperCreator
import io.tswf.minecraft.blockventure.core.view.ViewNetworkState
import io.tswf.minecraft.blockventure.core.view.ViewNetworkStatePacketHandler
import net.minecraft.network.NetworkSide
import net.minecraft.network.NetworkState
import net.minecraft.network.PacketByteBuf
import net.minecraft.network.packet.Packet

import java.util.function.Function

@Slf4j
@CompileStatic
class TransmutationNetworkState {

    @Transmute(
            targetClass = @ClassReference(NetworkState),
            targetMethod = @MethodDescription(
                    name = "<clinit>",
                    descriptor = "()V"
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtExit
    static void registerBlockventurePlayNetworkPackets() {
        def playNetworkState = NetworkState.PLAY
        def viewPlayNetworkState = (ViewNetworkState) (Object) playNetworkState
        def packetHandlers = (Map<NetworkSide, ViewNetworkStatePacketHandler>) (Map) viewPlayNetworkState.packetHandlers

        for (def networkSide : NetworkSide.values()) {
            packetHandlers[networkSide]?.let { viewPacketHandler ->
                log.info("Registering ${NetworkPlayPacketWrapper.name} packet on side ${networkSide.name()}!")

                viewPacketHandler.callRegister(
                        NetworkPlayPacketWrapper as Class,
                        { PacketByteBuf buf -> NetworkPlayPacketWrapperCreator.createWrapperForPlayPacket(buf) } as Function<PacketByteBuf, Packet>
                )
            }
        }
    }
}
